@@ -3,7 +3,6 @@
  */
 
 #include <ctype.h>
-//#include <errno.h>
 #include <math.h>
 #include <R.h>
 #include <R_ext/BLAS.h>
@@ -26,11 +25,9 @@
 
 
 void poly_domain_nonnegative_1d(int a, int b, double c, int larger,
-								int *num_intervals, double **lefts_pt, double **rights_pt, int *errno_status){
+								int *num_intervals, double **lefts_pt, double **rights_pt){
 	// If requires x to be non-negative.
 	*num_intervals = 0; // Default
-	*lefts_pt = (double*)malloc(sizeof(double)); // Max num of intervals is 1
-	*rights_pt = (double*)malloc(sizeof(double));
 	if (b == 0) { // Special functions log and exp for b == 0
 		if (a == 0) { // a == 0, b == 0 -> log
 			*num_intervals = 1;
@@ -83,16 +80,16 @@ void poly_domain_nonnegative_1d(int a, int b, double c, int larger,
 	} else { // b != 0 and a != 0 and c > 0
 		*num_intervals = 1;
 		if ((a > 0) ^ larger) { // x^(a/b) < c for a > 0 or x^(a/b) > c for a < 0
-			**lefts_pt = 0; **rights_pt = frac_pow(c, b, a, false, TRUE, errno_status);
+			**lefts_pt = 0; **rights_pt = frac_pow(c, b, a, false, TRUE);
 		} else { // x^(a/b) > c for a > 0 or x^(a/b) < c for a < 0
-			**lefts_pt = frac_pow(c, b, a, false, TRUE, errno_status); **rights_pt = INFINITY;
+			**lefts_pt = frac_pow(c, b, a, false, TRUE); **rights_pt = INFINITY;
 		}
 	}
 	return;
 }
 
 void log_exp_domain_1d(int a, double c, int larger, int abs,
-					   int *num_intervals, double **lefts_pt, double **rights_pt, int *errno_status) {
+					   int *num_intervals, double **lefts_pt, double **rights_pt) {
 	if (a == 0) { // a == 0, b == 0 -> log
 		if (abs) { // log(|x|)
 			if (larger) { // log(|x|) > c
@@ -170,7 +167,7 @@ void log_exp_domain_1d(int a, double c, int larger, int abs,
 }
 
 void poly_domain_1d(int a, int b, double c, int larger, int abs, int nonnegative,
-					  int *num_intervals, double **lefts_pt, double **rights_pt, int *errno_status){
+					  int *num_intervals, double **lefts_pt, double **rights_pt){
 	/* Returns the intervals on which x^(a/b) >/< c (larger == 1 or 0), or abs(x) if abs == 1.
 	 Assumes b > 0, and a and b coprime. Ignores Lebesgue null sets and
 	 joins intervals whenever possible (i.e. (-INF,0),(0,INF) will be joined to (-INF,INF)).
@@ -178,17 +175,14 @@ void poly_domain_1d(int a, int b, double c, int larger, int abs, int nonnegative
 	*num_intervals = 0; // default setting; if error occurs returns with 0
 	*lefts_pt = (double*)malloc(2 * sizeof(double)); // Max num of intervals is 2, so preset for convenience
 	*rights_pt = (double*)malloc(2 * sizeof(double));
-	if (b < 0) {
-		*errno_status = 1;
-		Rprintf("!!!power denominator must be > 0!! Got %d!!!\n", b);
-		return;
-	}
+	if (b < 0)
+		error("In poly_domain_1d(): Power denominator must be > 0. Got %d.\n", b);
 	if (nonnegative) {
-		poly_domain_nonnegative_1d(a, b, c, larger, num_intervals, lefts_pt, rights_pt, errno_status);
+		poly_domain_nonnegative_1d(a, b, c, larger, num_intervals, lefts_pt, rights_pt);
 		return;
 	}
 	if (b == 0) { // Special functions log and exp for b == 0
-		log_exp_domain_1d(a, c, larger, abs, num_intervals, lefts_pt, rights_pt, errno_status);
+		log_exp_domain_1d(a, c, larger, abs, num_intervals, lefts_pt, rights_pt);
 		return;
 	}
 	if (a == 0) { // For b != 0, x^(a/b) always 1.0
@@ -206,7 +200,7 @@ void poly_domain_1d(int a, int b, double c, int larger, int abs, int nonnegative
 				**rights_pt = INFINITY;
 			}
 		} else { // b even, c > 0
-			double cba = frac_pow(c, b, a, false, TRUE, errno_status); // c^(b/a)
+			double cba = frac_pow(c, b, a, false, TRUE); // c^(b/a)
 			if (abs) {
 				if ((a > 0) ^ larger) { // |x| < C^(b/a): [-c^(b/a), c^(b/a)]
 					*num_intervals = 1;
@@ -234,7 +228,7 @@ void poly_domain_1d(int a, int b, double c, int larger, int abs, int nonnegative
 					**lefts_pt = -INFINITY; **rights_pt = INFINITY;
 				}
 			} else { // b odd, abs or a even, c > 0
-				double cba = frac_pow(c, b, a, false, TRUE, errno_status);
+				double cba = frac_pow(c, b, a, false, TRUE);
 				if ((a > 0) ^ larger) { // [-c^(b/a), c^(b/a)]
 					*num_intervals = 1;
 					**lefts_pt = -cba; **rights_pt = cba;
@@ -252,7 +246,7 @@ void poly_domain_1d(int a, int b, double c, int larger, int abs, int nonnegative
 					*num_intervals = 1; **lefts_pt = -INFINITY; **rights_pt = 0;
 				}
 			} else { // b odd, no abs and a odd, c != 0
-				double cba = frac_pow(c, b, a, false, TRUE, errno_status);
+				double cba = frac_pow(c, b, a, false, TRUE);
 				if (a > 0) { // b odd, no abs and a odd, c != 0, a > 0
 					*num_intervals = 1;
 					if (larger) { // x^(a/b) | c is simply x | c^(b/a): [c^(b/a), INF)
@@ -291,7 +285,7 @@ void domain_1d(const int *idx, const int *p, const double *x,
 			  const int *num_int_params, int *int_params,
 			  const int *num_double_params, double *double_params,
 			  int *num_intervals, double **lefts_pt, double **rights_pt,
-			  double **cache, int *errno_status){
+			  double **cache){
 	/*
 	 cache: For caching results common to all *idx to speed up calculation of h. Boundary type-specific. Defined as double** since each boundary type may need to allocate different size of memory.
 	 	uniform: Not used.
@@ -309,16 +303,10 @@ void domain_1d(const int *idx, const int *p, const double *x,
 	 	If int_params[2] is 1, pow0 = ... = pow(p-1) = int_params[4] / int_params[5]; otherwise, pow0 = int_params[4] / int_params[4+p], ... , pow(p-1) = int_params[3+p] / int_params[3+2*p].
 	 	If int_params[3] is 1, x will be restricted to the non-negative orthant.
 	 */
-	if (*num_char_params < 1) {
-		*errno_status = 1;
-		Rprintf("!!!In domain_1d(): Number of string parameters must be at least 1 and the first string must be the domain type!!!\n");
-		return;
-	}
-	if (*idx < 0 || *idx >= *p) {
-		*errno_status = 1;
-		Rprintf("!!!In domain_1d(): Invalid index %d. Should be between [0, %d]!!!\n", *idx, *p-1);
-		return;
-	}
+	if (*num_char_params < 1)
+		error("In domain_1d(): Number of string parameters must be at least 1 and the first string must be the domain type.\n");
+	if (*idx < 0 || *idx >= *p)
+		error("In domain_1d(): Invalid index %d. Should be between [0, %d].\n", *idx, *p-1);
 	if (strcmp(char_params[0], "R") == 0 || strcmp(char_params[0], "R+") == 0) {
 		*num_intervals = 1;
 		*lefts_pt = (double*)malloc(sizeof(double));
@@ -327,13 +315,16 @@ void domain_1d(const int *idx, const int *p, const double *x,
 		**rights_pt = INFINITY;
 		return;
 	} else if (strcmp(char_params[0], "uniform") == 0) {
-		if (*num_double_params % 2 || *num_double_params <= 0 || *num_int_params != 2) {
-			*errno_status = 1;
-			Rprintf("!!!In domain_1d(): For uniform boundaries, num_double_params must be an even positive number and double params should contain the left endpoints followed by the right endpoints, and num_int_params must be 2 with int_params[0] indicating whether lefts[0] is -Inf and int_params[1] whether rights[-1] is Inf!!!\n");
-			return;
-		}
+		if (*num_double_params % 2 || *num_double_params <= 0 || *num_int_params != 2)
+			error("In domain_1d(): For uniform boundaries, num_double_params must be an even positive number and double params should contain the left endpoints followed by the right endpoints, and num_int_params must be 2 with int_params[0] indicating whether lefts[0] is -Inf and int_params[1] whether rights[-1] is Inf.\n");
 		*num_intervals = *num_double_params / 2;
-		*lefts_pt = double_params; *rights_pt = double_params + *num_intervals;
+		*lefts_pt = (double*)malloc(*num_intervals * sizeof(double));
+		*rights_pt = (double*)malloc(*num_intervals * sizeof(double));
+		for (int j = 0; j < *num_intervals; j++) {
+			(*lefts_pt)[j] = double_params[j];
+			(*rights_pt)[j] = double_params[j + *num_intervals];
+		}
+		// *lefts_pt = double_params; *rights_pt = double_params + *num_intervals;
 		if (int_params[0])
 			(*lefts_pt)[0] = -INFINITY;
 		if (int_params[1])
@@ -345,18 +336,12 @@ void domain_1d(const int *idx, const int *p, const double *x,
 			sum_all = **cache;
 		else {
 			for (int j = 0; j < *p; j++) {
-				if (x[j] <= -TOL) { //// Or <= TOL?
-					*errno_status = 1;
-					Rprintf("!!!In domain_1d(): x[%d] = %f <= 0 not allowed for the simplex!!!\n", j, x[j]);
-					return;
-				}
+				if (x[j] <= -TOL) //// Or <= TOL?
+					error("In domain_1d(): x[%d] = %f <= 0 not allowed for the simplex.\n", j, x[j]);
 				sum_all += x[j];
 			}
-			if (sum_all >= 1) { //// Or >= 1 - TOL?
-				*errno_status = 1;
-				Rprintf("!!!In domain_1d(): sum(x) = %f >= 1 not allowed for the simplex!!!\n", sum_all);
-				return;
-			}
+			if (sum_all >= 1) //// Or >= 1 - TOL?
+				error("In domain_1d(): sum(x) = %f >= 1 not allowed for the simplex.\n", sum_all);
 		}
 		// Cache if needed
 		if (cache && *idx == 0) { // If need to cache sum of x
@@ -370,11 +355,8 @@ void domain_1d(const int *idx, const int *p, const double *x,
 		**rights_pt = 1 - (sum_all - x[*idx]);
 		return;
 	} else if (strcmp(char_params[0], "polynomial") == 0) {
-		if (*num_char_params != 2) {
-			*errno_status = 1;
-			Rprintf("!!!In domain_1d(): Number of string parameters must be 2 for polynomial type boundary, where the second string parameter is the rule for combining the boundaries defined by each polynomial!!!\n");
-			return;
-		}
+		if (*num_char_params != 2)
+			error("In domain_1d(): Number of string parameters must be 2 for polynomial type boundary, where the second string parameter is the rule for combining the boundaries defined by each polynomial.\n");
 		int num_eqs = *int_params++;
 		const char *postfix = char_params[1];
 		int *num_intervals_list = (int*)malloc(num_eqs * sizeof(int));
@@ -395,21 +377,22 @@ void domain_1d(const int *idx, const int *p, const double *x,
 				double budget = *double_params++;
 				int_params += 7;
 				poly_domain_1d(power_numer, power_denom, budget, larger, abs, nonneg,
-							   num_intervals_list + eq_i, lefts_list + eq_i, rights_list + eq_i, errno_status);
-				if (*errno_status) {
-					Rprintf("!!!In domain_1d(): Error occurred when calling poly_domain_1d() for (uniform) inequality %d. Trying to find domain for %sx%d%s^(%d/%d)%s%f (nonneg=%d)!!!\n", eq_i + 1, abs?"|":"", *idx + 1, abs?"|":"", power_numer, power_denom, larger?">":"<", budget, nonneg);
+							   num_intervals_list + eq_i, lefts_list + eq_i, rights_list + eq_i);
+				/*if (*errno_status) {
+					error("In domain_1d(): Error occurred when calling poly_domain_1d() for (uniform) inequality %d. Trying to find domain for %sx%d%s^(%d/%d)%s%f (nonneg=%d).\n", eq_i + 1, abs?"|":"", *idx + 1, abs?"|":"", power_numer, power_denom, larger?">":"<", budget, nonneg);
 					return;
-				}
+				}*/
 				for (int i = 0; i < *p; i++)
 					if (i != *idx) {
 						if (nonneg && x[i] < 0) { // If other components do not satisfy nonnegative requirement, domain for this component should also be empty
 							num_intervals_list[eq_i] = 0;
+							free(lefts_list[eq_i]); free(rights_list[eq_i]);
 							break;
 						}
-						double tmp = frac_pow(x[i], power_numer, power_denom, abs, FALSE, errno_status);
-						if (*errno_status || (tmp < budget && larger) || (tmp > budget && !larger)) {
-							*errno_status = 0;
+						double tmp = frac_pow(x[i], power_numer, power_denom, abs, FALSE);
+						if (isnan(tmp) || (tmp < budget && larger) || (tmp > budget && !larger)) {
 							num_intervals_list[eq_i] = 0;
+							free(lefts_list[eq_i]); free(rights_list[eq_i]);
 							break;
 						}
 					}
@@ -424,7 +407,6 @@ void domain_1d(const int *idx, const int *p, const double *x,
 					for (int i = 0; i < num_eqs; i++)
 						(*cache)[i] = NAN;
 				}
-						
 				
 				if (nonneg) {
 					int other_components_oob = 0;
@@ -446,14 +428,13 @@ void domain_1d(const int *idx, const int *p, const double *x,
 					power_numer_this = int_params[5]; power_denom_this = int_params[6];
 					if (cache && *idx != 0 && !isnan((*cache)[eq_i])) {// If pow sum already calculated (p != 0) and cached
 						sum_all = (*cache)[eq_i];
-						sum_other = sum_all - double_params[*idx] * frac_pow(x[*idx], power_numer_this, power_denom_this, abs, FALSE, errno_status);
-					}
-					else {
+						sum_other = sum_all - double_params[*idx] * frac_pow(x[*idx], power_numer_this, power_denom_this, abs, FALSE);
+					} else {
 						for (int j = 0; j < *p; j++)
 							if (j != *idx && double_params[j] != 0.0)
 								// Calculate sum_other and then sum_all so that even when x[*idx] is out of bound so that frac_pow returns NAN, we can still calculate its domain as long as frac_pow is valid for all other components; calculating sum_all and then sum_other will make both NAN in that case
-								sum_other += double_params[j] * frac_pow(x[j], int_params[5], int_params[6], abs, FALSE, errno_status); // FALSE: do not produce warning if power not well-defined
-						sum_all = sum_other + double_params[*idx] * frac_pow(x[*idx], power_numer_this, power_denom_this, abs, FALSE, errno_status);
+								sum_other += double_params[j] * frac_pow(x[j], int_params[5], int_params[6], abs, FALSE); // FALSE: do not produce warning if power not well-defined
+						sum_all = sum_other + double_params[*idx] * frac_pow(x[*idx], power_numer_this, power_denom_this, abs, FALSE);
 					}
 					int_params += 7;
 				} else { // Different powers
@@ -461,31 +442,28 @@ void domain_1d(const int *idx, const int *p, const double *x,
 					power_numer_this = int_params[*idx + 5]; power_denom_this = int_params[*idx + 5 + *p];
 					if (cache && *idx != 0 && !isnan((*cache)[eq_i])) {// If pow sum already calculated (p != 0) and cached
 						sum_all = (*cache)[eq_i];
-						sum_other = sum_all - double_params[*idx] * frac_pow(x[*idx], power_numer_this, power_denom_this, abs, FALSE, errno_status);
+						sum_other = sum_all - double_params[*idx] * frac_pow(x[*idx], power_numer_this, power_denom_this, abs, FALSE);
 					}
 					else {
 						for (int j = 0; j < *p; j++)
 							if (j != *idx && double_params[j] != 0.0){
 								reduce_gcd(int_params + 5 + j, int_params + 5 + *p + j); // Make the numer and denom coprime
 								// Calculate sum_other and then sum_all so that even when x[*idx] is out of bound so that frac_pow returns NAN, we can still calculate its domain as long as frac_pow is valid for all other components; calculating sum_all and then sum_other will make both NAN in that case
-								sum_other += double_params[j] * frac_pow(x[j], int_params[5 + j], int_params[5 + *p + j], abs, FALSE, errno_status); // FALSE: do not produce warning if power not well-defined
+								sum_other += double_params[j] * frac_pow(x[j], int_params[5 + j], int_params[5 + *p + j], abs, FALSE); // FALSE: do not produce warning if power not well-defined
 							}
-						sum_all = sum_other + double_params[*idx] * frac_pow(x[*idx], power_numer_this, power_denom_this, abs, FALSE, errno_status);
+						sum_all = sum_other + double_params[*idx] * frac_pow(x[*idx], power_numer_this, power_denom_this, abs, FALSE);
 					}
 					int_params += 5 + 2 * (*p);
 				}
 				if (cache && *idx == 0) { // If need to cache pow sum
 					(*cache)[eq_i] = sum_all; // Cache even if NaN otherwise other *idx might not know the current x is out of domain for inequality eq_i
 				}
-				if (*errno_status && !isnan(sum_other)) // If has errno_status but sum_other not NAN, an error occurred whenn calculating frac_pow for this index but not for the others, so can still proceed with finding the domain for this index
-					*errno_status = 0;
-				if (*errno_status || isnan(sum_other)) { // It is possible that error occurs in frac_pow for other some x[j] at the current x; this x may be generated from other inequalities with an OR rule, and becomes out of bound for the current inequality; simply set the domain for the current inequality to empty set. (E.g. x = [-1,-1,-1] generated from "x^(1/2)>1 || x < 0". Then the domain for x^(1/2)>1 for all components will be set to empty at x = [-1,-1,-1].)
-					*errno_status = 0;
+				if (isnan(sum_other)) { // It is possible that error occurs in frac_pow for other some x[j] at the current x; this x may be generated from other inequalities with an OR rule, and becomes out of bound for the current inequality; simply set the domain for the current inequality to empty set. (E.g. x = [-1,-1,-1] generated from "x^(1/2)>1 || x < 0". Then the domain for x^(1/2)>1 for all components will be set to empty at x = [-1,-1,-1].)
 					num_intervals_list[eq_i] = 0;
 					double_params += *p + 1;
 					continue;
 				}
-				
+				// Rprintf("eq_i=%d, double_params[*p]=%f, sum_all=%f, sum_other=%f, larger=%d\n", eq_i, double_params[*p], sum_all, sum_other, larger);
 				// Budget for this variable
 				double budget = double_params[*p] - sum_other;
 				// If coefficient is 0
@@ -507,111 +485,61 @@ void domain_1d(const int *idx, const int *p, const double *x,
 						budget = budget / double_params[*idx];
 					poly_domain_1d(power_numer_this, power_denom_this,
 							   budget, larger, abs, nonneg,
-							   num_intervals_list + eq_i, lefts_list + eq_i, rights_list + eq_i, errno_status);
-					if (*errno_status) {
+							   num_intervals_list + eq_i, lefts_list + eq_i, rights_list + eq_i);
+					/*if (*errno_status) {
 						Rprintf("!!!In domain_1d(): Error occurred when calling poly_domain_1d() for (non-uniform) inequality %d. Trying to find domain for %sx%d%s^(%d/%d)%s%f (nonneg=%d)!!!\n", eq_i + 1, abs?"|":"", *idx + 1, abs?"|":"", power_numer_this, power_denom_this, larger?">":"<", budget, nonneg);
 						return;
-					}
+					}*/
 				}
 				double_params += *p + 1;
 			}
 		}
+		/*for (int j = 0; j < *p; j++)
+			Rprintf("idx=%d: x[%d] = %f\n", *idx, j, x[j]);
+		for (int eq_i = 0; eq_i < num_eqs; eq_i++) {
+			Rprintf("Pre dm(): num_intervals_list[%d]: %d\n", eq_i, num_intervals_list[eq_i]);
+			if (num_intervals_list[eq_i])
+				Rprintf("Pre dm(): lefts_list[%d]: %p  rights_list[%d]: %p\n", eq_i, (lefts_list[eq_i]), eq_i, (rights_list[eq_i]));
+		}*/
+		
 		// Evaluates the domain using domains for each inequality with intersections/unions
 		evaluate_logic(&num_eqs, postfix, num_intervals_list, lefts_list,
-					   rights_list, num_intervals, lefts_pt, rights_pt, errno_status);
+					   rights_list, num_intervals, lefts_pt, rights_pt);
 		
 		/*for (int i = 0; i < num_eqs; i++) {
 			Rprintf("\nSubinterval for x%d in eq %d: ", *idx, i);
 			for (int j = 0; j < num_intervals_list[i]; j++)
 				Rprintf("%d, [%f, %f]; ", j, lefts_list[i][j], rights_list[i][j]);
-		}////*/
+		}*/
 		
+		for (int eq_i = 0; eq_i < num_eqs; eq_i++)
+			if (num_intervals_list[eq_i]) {
+				//Rprintf("dm(): lefts_list[%d]: %p  rights_list[%d]: %p\n", eq_i, (lefts_list[eq_i]), eq_i, (rights_list[eq_i])); ////
+				free(lefts_list[eq_i]); free(rights_list[eq_i]);
+			}
 		free(num_intervals_list); free(lefts_list); free(rights_list);
 		return;
-	} else {
-		*errno_status = 1;
-		Rprintf("!!!In domain_1d(): domain type %s not supported!!!\n", char_params[0]);
-		return;
-	}
+	} else
+		error("In domain_1d(): domain type %s not supported.\n", char_params[0]);
 }
 
 
-void poly_domain_1d_for_R(int *a, int *b, double *c, int *larger, int *abs, int *nonnegative, int *num_intervals, double *lefts, double *rights, int *print, int *errno_status){
+void poly_domain_1d_for_R(int *a, int *b, double *c, int *larger, int *abs, int *nonnegative, int *num_intervals, double *lefts, double *rights, int *print){
 	// Prints and returns 1d domain for a domain specified by a polynomial x^(a/b) >/< c, used for calls from R. Assumes lefts and rights have been allocated at least 2 elements since the max number of intervals is 2.
 	double *lefts0, *rights0;
-	poly_domain_1d(*a, *b, *c, *larger, *abs, *nonnegative, num_intervals, &lefts0, &rights0, errno_status);
-	if (*errno_status) {
-		Rprintf("!!!Error occurred when calling poly_domain_1d in poly_domain_1d_for_R()!!!\n");
-		return;
-	}
+	poly_domain_1d(*a, *b, *c, *larger, *abs, *nonnegative, num_intervals, &lefts0, &rights0);
 	for (int i = 0; i < *num_intervals; i++) {
 		if (*print)
-			Rprintf("Interval %d: [%f, %f]\n", i, lefts0[i], rights0[i]);
+			Rprintf("In poly_domain_1d_for_R(): Interval %d: [%f, %f]\n", i, lefts0[i], rights0[i]);
 		lefts[i] = lefts0[i]; rights[i] = rights0[i];
 	}
+	free(lefts0); free(rights0);
 }
 
-/*
-void h(const int *p, const double *x, double *hx, double *hpx, const double *h_pow, const double *h_trunc,
-	   const int *num_char_params, const char **char_params,
-	   const int *num_int_params, int *int_params,
-	   int *num_double_params, double *double_params, int *errno_status){
-	double *lefts, *rights;
-	double **cache = (double**)malloc(sizeof(double*)); // Caching results common to all idx
-	int num_intervals;
-	if (*h_pow < 0) {*errno_status = 1; Rprintf("!!!h_pow must be >= 0!!!\n"); return;}
-	if (*h_trunc <= 0) {*errno_status = 1; Rprintf("!!!h_trunc must be > 0!!!\n"); return;}
-	for (int idx = 0; idx < *p; idx++) {
-		domain_1d(&idx, p, x, num_char_params, char_params,
-			   num_int_params, int_params, num_double_params, double_params,
-			   &num_intervals, &lefts, &rights, cache, errno_status);
-		if (*errno_status) {
-			Rprintf("!!!Error ocurred when calling domain_1d() in h()!!!\n");
-			return;
-		}
-		int bin = search_unfused(lefts, rights, num_intervals, x[idx], errno_status);
-		if (*errno_status) {
-			Rprintf("!!!Error occurred when calling search_unfused() in h()!!!\n");
-			return;
-		}
-		if (bin == -1) {
-			*errno_status = 1;
-			Rprintf("!!!h: x given not in domain!!!\n");
-			return;
-		}
-		hpx[idx] = 0;
-		if (fabs(*h_pow) < TOL) {
-			// If power 0 used for h (allowed for some distributions), return min(1, trunc);
-			// Steps above are still carried out as a sanity check if the given point
-			// is inside the boundary.
-			hx[idx] = fmin(1, *h_trunc);
-			return;
-		}
-		hx[idx] = *h_trunc;
-		if (lefts[bin] != -INFINITY && x[idx] - lefts[bin] < hx[idx]) {
-			hx[idx] = x[idx] - lefts[bin];
-			hpx[idx] = 1;
-		}
-		if (rights[bin] != INFINITY && rights[bin] - x[idx] < hx[idx]) {
-			hx[idx] = rights[bin] - x[idx];
-			hpx[idx] = -1;
-		}
-		if (hpx[idx] != 0) {
-			if (hx[idx] > 0) // If x not at boundary: a(x-x0)^(a-1) or -a(x0-x)^(a-1)
-				hpx[idx] *= *h_pow * pow(hx[idx], *h_pow - 1);
-			else
-				hpx[idx] = 0; // at boundary
-		}
-		hx[idx] = pow(hx[idx], *h_pow); // |x-x0|^(a-1)
-	}
-	free(cache);
-}
-*/
-
 void dist(const int *n, const int *p, const double *x, double *dists, int *dist_ps,
 	   const int *num_char_params, const char **char_params,
 	   const int *num_int_params, int *int_params,
-	   int *num_double_params, double *double_params, int *errno_status){
+	   int *num_double_params, double *double_params){
 	/*
 	 x: of size p (num parameters) * n (num samples)
 	 dists: of size p * n
@@ -624,23 +552,9 @@ void dist(const int *n, const int *p, const double *x, double *dists, int *dist_
 		for (int idx = 0; idx < *p; idx++) {
 			domain_1d(&idx, p, x, num_char_params, char_params,
 					  num_int_params, int_params, num_double_params, double_params,
-					  &num_intervals, &lefts, &rights, cache, errno_status);
-			if (*errno_status) {
-				Rprintf("!!!Error ocurred when calling domain_1d() in h()!!!\n");
-				return;
-			}
-			
-			int bin = search_unfused(lefts, rights, num_intervals, x[idx], errno_status);
-			if (*errno_status) {
-				Rprintf("!!!Error occurred when calling search_unfused() in h()!!!\n");
-				return;
-			}
-			if (bin == -1) {
-				*errno_status = 1;
-				Rprintf("!!!h: x given not in domain!!!\n");
-				return;
-			}
-			
+					  &num_intervals, &lefts, &rights, cache);
+
+			int bin = search_unfused(lefts, rights, num_intervals, x[idx]);
 			dist_ps[idx] = 0; // If domain is R, will return 0 derivative
 			dists[idx] = INFINITY; // If domain is R, will return INF distance
 			if (lefts[bin] != -INFINITY && x[idx] - lefts[bin] < dists[idx]) {
@@ -658,7 +572,10 @@ void dist(const int *n, const int *p, const double *x, double *dists, int *dist_
 			}
 			if (dists[idx] < TOL) // If at boundary, derivative 0
 				dist_ps[idx] = 0;
+			free(lefts); free(rights);
 		}
+		if (strcmp(char_params[0], "simplex") == 0 || strcmp(char_params[0], "polynomial") == 0)
+			free(*cache);
 		free(cache);
 		x += *p; dists += *p; dist_ps += *p;
 	}
@@ -666,7 +583,7 @@ void dist(const int *n, const int *p, const double *x, double *dists, int *dist_
 
 
 void fuse_endpoints(const int *num_intervals, const double *lefts, const double *rights,
-					double *fused, double *disp, int *errno_status)
+					double *fused, double *disp)
 /* to fuse intervals with endpoints specified by lefts and rights, left-aligned;
  [x1, x2], [x3, x4], [x5, x6] becomes {x1, x2, x2+x4-x3, x2+x4-x3+x6-x5}
  *num_intervals: number of intervals
@@ -680,10 +597,8 @@ void fuse_endpoints(const int *num_intervals, const double *lefts, const double
 	fused[0] = lefts[0];
 	fused[1] = rights[0];
 	disp[0] = 0.0;
-	if (*num_intervals < 1) {
-		*errno_status = 1;
-		Rprintf("!!!In fuse_endpoints: number of intervals < 1!!!\n");
-	}
+	if (*num_intervals < 1)
+		error("In fuse_endpoints: number of intervals < 1.\n");
 	for (int interval = 1; interval < *num_intervals; interval++){
 		fused[1 + interval] = fused[interval] + rights[interval] - lefts[interval];
 		disp[interval] = disp[interval - 1] + lefts[interval] - rights[interval - 1];
@@ -692,7 +607,7 @@ void fuse_endpoints(const int *num_intervals, const double *lefts, const double
 
 /* ********************************************************************* */
 
-int binarySearch_fused(const double *arr, int l, int r, double x, int *errno_status)
+int binarySearch_fused(const double *arr, int l, int r, double x)
 /* Binary search for the left endpoint of the FUSED interval x belongs to.
  Assumes arr has at least 2 elements and x is inside the range of arr.
  *arr: an array
@@ -704,13 +619,13 @@ int binarySearch_fused(const double *arr, int l, int r, double x, int *errno_sta
 	if (r > l + 1) {
 		int mid = (l + r) / 2;
 		if (arr[mid] >= x)
-			return binarySearch_fused(arr, l, mid, x, errno_status);
-		return binarySearch_fused(arr, mid, r, x, errno_status);
+			return binarySearch_fused(arr, l, mid, x);
+		return binarySearch_fused(arr, mid, r, x);
 	}
 	return l;
 }
 
-int naiveSearch_fused(const double *arr, int length, double x, int *errno_status)
+int naiveSearch_fused(const double *arr, int length, double x)
 /* Naive search for the left endpoint of the FUSED interval x belongs to.
  Assumes arr has at least 2 elements and x is inside the range of arr.
  *arr: an array
@@ -721,12 +636,11 @@ int naiveSearch_fused(const double *arr, int length, double x, int *errno_status
 	for (int i = 1; i <= length - 1; i++)
 		if (arr[i] >= x)
 			return i - 1;
-	*errno_status = 1;
-	Rprintf("!!!search_fused: %f not in fused domain!!!\n", x);
+	error("In search_fused(): %f not in fused domain.\n", x);
 	return -1;
 }
 
-int search_fused(const double *arr, int length, double x, int *errno_status)
+int search_fused(const double *arr, int length, double x)
 /* to return the index of left endpoint of the FUSED interval x belongs to using
  naive search; i.e. find smallest i s.t. arr[i] <= x <= arr[i+1]
  (if x is at the boundaries the smaller index is picked).
@@ -736,16 +650,13 @@ int search_fused(const double *arr, int length, double x, int *errno_status)
  x: the number to be searched
  */
 {
-	if (length < 2 || x < arr[0] || x > arr[length - 1]){
-		*errno_status = 1;
-		Rprintf("!!!search_fused: %f not in fused domain!!!\n", x);
-		return -1;
-	}
-	return (length > 8) ? binarySearch_fused(arr, 0, length - 1, x, errno_status) : naiveSearch_fused(arr, length, x, errno_status);
+	if (length < 2 || x < arr[0] || x > arr[length - 1])
+		error("In search_fused(): %f not in fused domain.\n", x);
+	return (length > 8) ? binarySearch_fused(arr, 0, length - 1, x) : naiveSearch_fused(arr, length, x);
 }
 
 double translate_unfuse(double x, int num_intervals, const double *fused,
-						const double *disp, int *errno_status)
+						const double *disp)
 /* translates x (relative the fused intervals) back to the x in the original domain.
  Example: [0, 1], [3, 6], [10, 15], [21, 28] is fused into {0,1,4,9,16};
  given x = 5, search for the bin it belongs to ([4,9]) and add back 6 (since
@@ -756,16 +667,11 @@ double translate_unfuse(double x, int num_intervals, const double *fused,
  *disp: displacements of intervals after fusion, an array of length num_intervals
  */
 {	if (num_intervals == 1) return x;  // No translation required
-	int bucket = search_fused(fused, num_intervals + 1, x, errno_status);
-	if (bucket == -1 || *errno_status == 1) {
-		*errno_status = 1;
-		Rprintf("!!!In translate_unfuse(): error occurred when calling search_fused(). Unable to find a bin for %f!!!\n", x);
-		return fused[0] - 1; // Return an invalid value
-	}
+	int bucket = search_fused(fused, num_intervals + 1, x);
 	return x + disp[bucket];
 }
 
-int search_unfused(const double *lefts, const double *rights, int length, double x, int *errno_status)
+int search_unfused(const double *lefts, const double *rights, int length, double x)
 /* to return the index of left endpoint of the ORIGINAL interval x belongs to using
  naive search; i.e. find the unique i s.t. lefts[i] <= x <= rights[i]. Uses naive search.
  *lefts: left bounds, an array of length num_intervals
@@ -774,28 +680,21 @@ int search_unfused(const double *lefts, const double *rights, int length, double
  x: the number to be searched
  */
 {
-	if (length < 1 || x < lefts[0] || x > rights[length - 1]) {
-		*errno_status = 1;
-		Rprintf("!!!search_unfused: %f not in domain. lefts[0] = %f, rights[%d] = %f. If %f is the finite_infinity you set in make_domain(), please increase the value!!!\n", x, lefts[0], length, rights[length-1], rights[length-1]);
-		return -1;
-	}
+	if (length < 1 || x < lefts[0] || x > rights[length - 1])
+		error("In search_unfused(): %f not in domain. lefts[0] = %f, rights[%d] = %f. If %f is the finite_infinity you set in make_domain(), please increase the value.\n", x, lefts[0], length, rights[length-1], rights[length-1]);
 	for (int i = length - 1; i >= 0; i--)
 		if (lefts[i] <= x) {
 			if (rights[i] >= x)
 				return i;
-			else {
-				*errno_status = 1;
-				Rprintf("!!!search_unfused: %f not in domain!!!\n", x);
-				return -1;
-			}
+			else
+				error("In search_unfused(): %f not in domain.\n", x);
 		}
-	*errno_status = 1;
-	Rprintf("!!!search_unfused: %f not in domain!!!\n", x);
+	error("In search_unfused(): %f not in domain.\n", x);
 	return -1;
 }
 
 double translate_fuse(double x, int num_intervals, const double *lefts,
-					  const double *rights, const double *disp, int *errno_status)
+					  const double *rights, const double *disp)
 /* translates x in the original domain to its position relative to the fused intervals.
  Example: [0, 1], [3, 6], [10, 15], [21, 28] is fused into {0,1,4,9,16};
  given x = 11, search for the bin it belongs to ([10,15]) and subtract 6 (since
@@ -807,10 +706,6 @@ double translate_fuse(double x, int num_intervals, const double *lefts,
  *disp: displacements of intervals after fusion, an array of length num_intervals
  */
 {	if (num_intervals == 1) return x;  // No translation required
-	int bucket = search_unfused(lefts, rights, num_intervals, x, errno_status);
-	if (*errno_status || bucket == -1) {
-		Rprintf("!!!Error occurred when calling search_unfused() in translate_fuse()!!!\n");
-		return lefts[0] - 1; // Return an invalid value
-	}
+	int bucket = search_unfused(lefts, rights, num_intervals, x);
 	return x - disp[bucket];
 }
